require File.expand_path '../../test_helper.rb', __FILE__

require 'crusade/gcm/push_notification'

require 'fixtures'
require 'webmock/minitest'

describe Crusade::GCM::PushNotification do
  let(:described_class) { Crusade::GCM::PushNotification }

  let(:configuration) { Fixtures.default_configuration }
  subject { described_class.new configuration }
  let(:fixture) { Fixtures.access_token 'success' }

  let(:notification) do
    Crusade::GCM::Notification.new(title: 'the title', body: 'the body', 
                                   channel_id: '123', subchannel_id: '456')
  end

  let(:access_token_stub) do
    stub_request(:post, "https://accounts.google.com/o/oauth2/token").with({
      headers: { 
        'Content-Type' => 'application/x-www-form-urlencoded'
      },
      params: { client_id: '1111/client', 
                client_secret: '"bbbbbbbbbb"', 
                refresh_token: "aaaaaaaabbbbbbb", grant_type: 'refresh_token' }
    }).to_return(status: [ 200, 'ok'], body: fixture.to_json)
  end

  let(:notification_stub) { notification_request_stub }
  
  def notification_request_stub
    stub_request(:post, "https://www.googleapis.com/gcm_for_chrome/v1/messages").with({
      headers: { 
        'Content-Type' => 'application/json', 
        'Authorization' => 'Bearer 1/fFBGRNJru1FQd44AzqT3Zg',
      },
      body: { channelId: '123', subchannelId: '456', 
              payload: '{"title":"the title","body":"the body"}'}
    }).to_return(status: [ 204, 'No Content'])
  end

  describe 'one request' do

    before do
      configuration.token_cache.cache.delete :access_token
    end

    before do
      access_token_stub
      notification_stub
    end

    it 'sends the notification' do
      subject.perform notification

      assert_requested access_token_stub
      assert_requested notification_stub
    end
  end

  describe 'multiple request' do

    before do
      configuration.token_cache.cache.delete :access_token
    end

    before do
      access_token_stub
      notification_stub
    end

    it 'sends the notification' do
      subject.perform notification
      WebMock.reset!
      notification_request_stub

      subject.perform notification

      assert_not_requested access_token_stub
      assert_requested notification_stub
    end
  end
end