require File.expand_path '../../test_helper.rb', __FILE__

require 'crusade/gcm/access_token'
require 'crusade/gcm/access_token_fetcher'

describe Crusade::GCM::AccessToken do
  class TestCache
    def initialize
      @cache = {}
    end

    def read(key)
      @cache[:access_token]
    end

    def write(key, value, opts = {})
      @cache[:access_token] = value
    end

    def clear
      @cache = {}
    end
  end

  let(:configuration)   { OpenStruct.new(token_cache: TestCache.new ) }
  let(:described_class) { Crusade::GCM::AccessToken }
  subject { described_class.new configuration }

  describe 'get' do
    let(:cached_token) { 'aaaaaa' }
    let(:new_token)    { "1/fFBGRNJru1FQd44AzqT3Zg" }

    describe 'a cached token is available' do
      before do
        configuration.token_cache.write(:access_token, cached_token)
      end

      it 'returns the cached value if available' do
        subject.get.must_equal cached_token
      end
    end

    describe 'cached value unavailable' do
      let(:fetcher) { OpenStruct.new(request_token: fetched)} 
      let(:fetched) { Fixtures.access_token 'success' }

      before do
        Crusade::GCM::AccessTokenFetcher.stub(:new, fetcher) do
          @token = subject.get
        end
      end

      it 'fetches a new token' do
        @token.must_equal new_token
      end

      it 'saves the value in the cache' do
        configuration.token_cache.read(:access_token).must_equal new_token
      end
    end
  end
end