require File.expand_path '../../../test_helper.rb', __FILE__

require 'crusade/gcm/cache/in_memory_cache'

describe Crusade::GCM::InMemoryCache do
  subject { Crusade::GCM::InMemoryCache }

  let(:key)   { 'a_key' }
  let(:value) { 'a value' }

  after do
    subject.cache.delete(key)
  end

  describe 'read' do
    before do
      subject.cache[key] = { value: value, valid_until: (Time.now + 10) }
    end

    it 'returns the value if the cache contains an entry for the key' do
      subject.read(key).must_equal value
    end

    it 'returns nil if the cache does not contain an entry for the key' do
      subject.read('a other key').must_be_nil
    end

    it 'returns nil if the cache contains an entry for the key but is outdated' do
      subject.cache[key] = { value: value, valid_until: (Time.now - 10) }

      subject.read(key).must_be_nil
    end
  end

  describe 'write' do
    let(:valid_until) { Time.now + 10 }

    before do
      subject.write(key, value)
    end

    it 'sets an entry in the cache' do
      subject.cache[key][:value].must_equal value
    end

    it 'sets default valid_until' do
      Integer(subject.cache[key][:valid_until]).must_be_close_to (Time.now.to_i + 3600), 1
    end

    it 'sets a valid_until if a expires_in provided' do
      subject.write(key, value, expires_in: 30)

      Integer(subject.cache[key][:valid_until]).must_be_close_to (Time.now.to_i + 30), 1
    end
  end
end