require File.expand_path '../../test_helper.rb', __FILE__

require 'crusade/notification'
require 'crusade/gcm/notification'
require 'json'

describe Crusade::GCM::Notification do
  let(:described_class) { Crusade::GCM::Notification }
  let(:notification)    { Crusade::Notification.new attrs }
  let(:channel_id)      { '1234a' }
  let(:subchannel_id)   { 'abcde' }

  let(:attrs) do
    {
      body: 'the body',
      title: 'the title',
      channel_id: channel_id,
      subchannel_id: subchannel_id
    }
  end

  describe 'initialize' do
    describe 'with a generic notification' do
      subject { described_class.new notification, channel_id, subchannel_id }

      it 'delegates the generic properties' do
        subject.body.must_equal 'the body'
      end

      it 'has a channel id' do
        subject.channel_id.must_equal channel_id
      end

      it 'has a subchannel id' do
        subject.subchannel_id.must_equal subchannel_id
      end
    end

    describe 'with a hash of attributes' do
      subject { described_class.new attrs }

      it 'internally builds a generic notification' do
        subject.body.must_equal 'the body'
      end

      it 'has a channel id' do
        subject.channel_id.must_equal channel_id
      end

      it 'has a subchannel id' do
        subject.subchannel_id.must_equal subchannel_id
      end
    end
  end

  describe 'payload and co' do
    subject { described_class.new attrs }

    let(:parsed_payload) { JSON.parse(subject.to_json) }
    let(:expected_payload) do
      {
        'channelId'    =>  channel_id,
        'subchannelId' =>  subchannel_id,
        'payload'      =>  '{"title":"the title","body":"the body"}'
      }
    end

    it { parsed_payload.must_equal expected_payload }
  end
end